@@ -6,12 +6,11 @@ import logging
 import traceback
 import moto.cloudformation.utils
 from six import iteritems
-from six import string_types
+from localstack.services.s3 import s3_listener
+from localstack.services.awslambda.lambda_api import get_handler_file_from_name
 from localstack.utils import common
 from localstack.utils.aws import aws_stack
-from localstack.services.s3 import s3_listener
 from localstack.utils.testutil import create_zip_file
-from localstack.services.awslambda.lambda_api import get_handler_file_from_name
 
 ACTION_CREATE = 'create'
 ACTION_DELETE = 'delete'
@@ -122,6 +121,25 @@ def sns_subscription_params(params, **kwargs):
     return result
 
 
+def events_put_rule_params(params, **kwargs):
+    attrs = ['ScheduleExpression', 'EventPattern', 'State', 'Description', 'Name']
+    result = select_parameters(*attrs)(params, **kwargs)
+    result['Name'] = result.get('Name') or PLACEHOLDER_RESOURCE_NAME
+
+    def wrap_in_lists(o, **kwargs):
+        if isinstance(o, dict):
+            for k, v in o.items():
+                if not isinstance(v, (dict, list)):
+                    o[k] = [v]
+        return o
+
+    pattern = result.get('EventPattern')
+    if isinstance(pattern, dict):
+        wrapped = common.recurse_object(pattern, wrap_in_lists)
+        result['EventPattern'] = json.dumps(wrapped)
+    return result
+
+
 def es_add_tags_params(params, **kwargs):
     es_arn = aws_stack.es_domain_arn(params.get('DomainName'))
     tags = params.get('Tags', [])
@@ -212,12 +230,17 @@ RESOURCE_TO_FUNCTION = {
             'function': 'put_bucket_notification_configuration',
             'parameters': s3_bucket_notification_config
         }],
-        'delete': {
+        'delete': [{
+            'function': 'delete_bucket_policy',
+            'parameters': {
+                'Bucket': 'PhysicalResourceId'
+            }
+        }, {
             'function': 'delete_bucket',
             'parameters': {
                 'Bucket': 'PhysicalResourceId'
             }
-        }
+        }]
     },
     'S3::BucketPolicy': {
         'create': {
@@ -230,8 +253,9 @@ RESOURCE_TO_FUNCTION = {
             'function': 'create_queue',
             'parameters': {
                 'QueueName': ['QueueName', PLACEHOLDER_RESOURCE_NAME],
-                'Attributes': params_select_attributes('DelaySeconds', 'MaximumMessageSize',
-                    'MessageRetentionPeriod', 'VisibilityTimeout', 'RedrivePolicy'),
+                'Attributes': params_select_attributes('ContentBasedDeduplication', 'DelaySeconds', 'FifoQueue',
+                                                       'MaximumMessageSize', 'MessageRetentionPeriod',
+                                                       'VisibilityTimeout', 'RedrivePolicy'),
                 'tags': params_list_to_dict('Tags')
             }
         },
@@ -389,13 +413,7 @@ RESOURCE_TO_FUNCTION = {
     'Events::Rule': {
         'create': [{
             'function': 'put_rule',
-            'parameters': {
-                'Name': PLACEHOLDER_RESOURCE_NAME,
-                'ScheduleExpression': 'ScheduleExpression',
-                'EventPattern': 'EventPattern',
-                'State': 'State',
-                'Description': 'Description'
-            }
+            'parameters': events_put_rule_params
         }, {
             'function': 'put_targets',
             'parameters': {
@@ -403,7 +421,14 @@ RESOURCE_TO_FUNCTION = {
                 'EventBusName': 'EventBusName',
                 'Targets': 'Targets'
             }
-        }]
+        }],
+        'delete': {
+            'function': 'delete_rule',
+            'parameters': {
+                'Name': 'PhysicalResourceId'
+            }
+
+        }
     },
     'IAM::Role': {
         'create': {
@@ -415,6 +440,12 @@ RESOURCE_TO_FUNCTION = {
                             'Description', 'MaxSessionDuration', 'PermissionsBoundary', 'Tags'),
                         'AssumeRolePolicyDocument'),
                     {'RoleName': PLACEHOLDER_RESOURCE_NAME})
+        },
+        'delete': {
+            'function': 'delete_role',
+            'parameters': {
+                'RoleName': 'PhysicalResourceId'
+            }
         }
     },
     'ApiGateway::RestApi': {
@@ -499,6 +530,12 @@ RESOURCE_TO_FUNCTION = {
                 'definition': 'DefinitionString',
                 'roleArn': lambda params, **kwargs: get_role_arn(params.get('RoleArn'), **kwargs)
             }
+        },
+        'delete': {
+            'function': 'delete_state_machine',
+            'parameters': {
+                'stateMachineArn': 'PhysicalResourceId'
+            }
         }
     },
     'StepFunctions::Activity': {
@@ -680,7 +717,7 @@ def retrieve_resource_details(resource_id, resource_status, resources, stack_nam
                 raise Exception('ResourceNotFound')
             return mapping[0]
         elif resource_type == 'IAM::Role':
-            role_name = resource_props.get('RoleName') or resource_id
+            role_name = resource_props.get('RoleName')
             role_name = resolve_refs_recursively(stack_name, role_name, resources)
             return aws_stack.connect_to_service('iam').get_role(RoleName=role_name)['Role']
         elif resource_type == 'SSM::Parameter':
@@ -774,8 +811,11 @@ def retrieve_resource_details(resource_id, resource_status, resources, stack_nam
             bucket_name = resolve_refs_recursively(stack_name, bucket_name, resources)
             return aws_stack.connect_to_service('s3').get_bucket_policy(Bucket=bucket_name)
         elif resource_type == 'Logs::LogGroup':
-            # TODO implement
-            raise Exception('ResourceNotFound')
+            group_name = resource_props.get('LogGroupName')
+            group_name = resolve_refs_recursively(stack_name, group_name, resources)
+            logs = aws_stack.connect_to_service('logs')
+            groups = logs.describe_log_groups(logGroupNamePrefix=group_name)['logGroups']
+            return ([g for g in groups if g['logGroupName'] == group_name] or [None])[0]
         elif resource_type == 'Kinesis::Stream':
             stream_name = resolve_refs_recursively(stack_name, resource_props['Name'], resources)
             result = aws_stack.connect_to_service('kinesis').describe_stream(StreamName=stream_name)
@@ -1027,6 +1067,10 @@ def deploy_resource(resource_id, resources, stack_name):
 
 
 def delete_resource(resource_id, resources, stack_name):
+    for res_id, res in resources.items():
+        if res['ResourceType'] == 'AWS::S3::Bucket':
+            s3_listener.remove_bucket_notification(res['PhysicalResourceId'])
+
     return execute_resource_action(resource_id, resources, stack_name, ACTION_DELETE)
 
 
@@ -1166,42 +1210,12 @@ def configure_resource_via_sdk(resource_id, resources, resource_type, func_detai
     return result
 
 
-# TODO remove?
-def deploy_template(template, stack_name):
-    if isinstance(template, string_types):
-        template = parse_template(template)
-
-    resource_map = template.get('Resources')
-    if not resource_map:
-        LOG.warning('CloudFormation template contains no Resources section')
-        return
-
-    next = resource_map
-
-    iters = 10
-    for i in range(0, iters):
-
-        # get resource details
-        for resource_id, resource in next.items():
-            stack_resource = describe_stack_resource(stack_name, resource_id)
-            resource['__details__'] = stack_resource
-
-        next = resources_to_deploy_next(resource_map, stack_name)
-        if not next:
-            return
-
-        for resource_id, resource in next.items():
-            deploy_resource(resource_id, resource_map, stack_name=stack_name)
-
-    LOG.warning('Unable to resolve all dependencies and deploy all resources ' +
-        'after %s iterations. Remaining (%s): %s' % (iters, len(next), next))
-
-
 def delete_stack(stack_name, stack_resources):
     resources = dict([(r['LogicalResourceId'], common.clone_safe(r)) for r in stack_resources])
     for key, resource in resources.items():
         resources[key]['Properties'] = common.clone_safe(resource)
-    for resource_id, resource in resources.items():
+
+    for resource_id in resources.keys():
         delete_resource(resource_id, resources, stack_name)
 
 
@@ -1252,6 +1266,7 @@ def all_dependencies_satisfied(resources, stack_name, all_resources, depending_r
     for resource_id, resource in iteritems(resources):
         if is_deployable_resource(resource):
             if not is_deployed(resource_id, all_resources, stack_name):
+                LOG.debug('Dependency for resource %s not yet deployed: %s' % (depending_resource, resource_id))
                 return False
     return True
 
