@@ -52,7 +52,7 @@ mutex_clean = threading.Semaphore(1)
 
 # misc. constants
 TIMESTAMP_FORMAT = '%Y-%m-%dT%H:%M:%S'
-TIMESTAMP_FORMAT_MILLIS = '%Y-%m-%dT%H:%M:%S.%fZ'
+TIMESTAMP_FORMAT_MICROS = '%Y-%m-%dT%H:%M:%S.%fZ'
 CODEC_HANDLER_UNDERSCORE = 'underscore'
 
 # chunk size for file downloads
@@ -160,6 +160,9 @@ class ShellCommandThread(FuncThread):
         # (Error: libc.musl-x86_64.so.1: cannot open shared object file)
         import psutil
 
+        if getattr(self, 'stopped', False):
+            return
+
         if not self.process:
             LOG.warning("No process found for command '%s'" % self.cmd)
             return
@@ -174,6 +177,7 @@ class ShellCommandThread(FuncThread):
         except Exception:
             if not quiet:
                 LOG.warning('Unable to kill process with pid %s' % parent_pid)
+        self.stopped = True
 
 
 class JsonObject(object):
@@ -201,8 +205,8 @@ class JsonObject(object):
         return result
 
     @classmethod
-    def from_json_list(cls, l):
-        return [cls.from_json(j) for j in l]
+    def from_json_list(cls, json_list):
+        return [cls.from_json(j) for j in json_list]
 
     @classmethod
     def as_dict(cls, obj):
@@ -392,7 +396,7 @@ def is_port_open(port_or_url, http_path=None, expect_success=True, protocols=['t
         return True
     url = '%s://%s:%s%s' % (protocol, host, port, http_path)
     try:
-        response = safe_requests.get(url)
+        response = safe_requests.get(url, verify=False)
         return not expect_success or response.status_code < 400
     except Exception:
         return False
@@ -408,12 +412,21 @@ def wait_for_port_open(port, http_path=None, expect_success=True, retries=10, sl
     return retry(check, sleep=sleep_time, retries=retries)
 
 
-def get_free_tcp_port():
-    tcp = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
-    tcp.bind(('', 0))
-    addr, port = tcp.getsockname()
-    tcp.close()
-    return port
+def get_free_tcp_port(blacklist=None):
+    blacklist = blacklist or []
+    for i in range(10):
+        tcp = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+        tcp.bind(('', 0))
+        addr, port = tcp.getsockname()
+        tcp.close()
+        if port not in blacklist:
+            return port
+    raise Exception('Unable to determine free TCP port with blacklist %s' % blacklist)
+
+
+def sleep_forever():
+    while True:
+        time.sleep(1)
 
 
 def get_service_protocol():
@@ -429,7 +442,9 @@ def timestamp(time=None, format=TIMESTAMP_FORMAT):
 
 
 def timestamp_millis(time=None):
-    return timestamp(time=time, format=TIMESTAMP_FORMAT_MILLIS)
+    microsecond_time = timestamp(time=time, format=TIMESTAMP_FORMAT_MICROS)
+    # truncating micorseconds to milliseconds. while leaving the Z
+    return microsecond_time[:-4] + microsecond_time[-1]
 
 
 def retry(function, retries=3, sleep=1, sleep_before=0, **kwargs):
@@ -626,7 +641,6 @@ def download(url, path, verify_ssl=True):
             return
         LOG.debug('Done downloading %s, response code %s, total bytes %d' % (url, r.status_code, total))
     finally:
-        LOG.debug('Cleaning up file handles for download of %s' % url)
         r.close()
         s.close()
 
@@ -694,6 +708,10 @@ def short_uid():
     return str(uuid.uuid4())[0:8]
 
 
+def long_uid():
+    return str(uuid.uuid4())
+
+
 def json_safe(item):
     """ return a copy of the given object (e.g., dict) that is safe for JSON dumping """
     try:
@@ -717,6 +735,10 @@ def fix_json_keys(item):
     return item_copy
 
 
+def canonical_json(obj):
+    return json.dumps(obj, sort_keys=True)
+
+
 def save_file(file, content, append=False):
     mode = 'a' if append else 'w+'
     if not isinstance(content, six.string_types):
@@ -754,8 +776,12 @@ def cleanup(files=True, env=ENV_DEV, quiet=True):
 
 
 def cleanup_threads_and_processes(quiet=True):
-    for t in TMP_THREADS:
-        t.stop(quiet=quiet)
+    for thread in TMP_THREADS:
+        if thread:
+            try:
+                thread.stop(quiet=quiet)
+            except Exception as e:
+                print(e)
     for p in TMP_PROCESSES:
         try:
             p.terminate()
@@ -766,9 +792,9 @@ def cleanup_threads_and_processes(quiet=True):
     clear_list(TMP_PROCESSES)
 
 
-def clear_list(l):
-    while len(l):
-        del l[0]
+def clear_list(list_obj):
+    while len(list_obj):
+        del list_obj[0]
 
 
 def cleanup_tmp_files():
@@ -898,6 +924,19 @@ def generate_ssl_cert(target_file=None, overwrite=False, random=False, return_co
     if target_file and not overwrite and os.path.exists(target_file):
         key_file_name = '%s.key' % target_file
         cert_file_name = '%s.crt' % target_file
+        try:
+            # extract key and cert from target_file and store into separate files
+            content = load_file(target_file)
+            key_start = '-----BEGIN PRIVATE KEY-----'
+            key_end = '-----END PRIVATE KEY-----'
+            cert_start = '-----BEGIN CERTIFICATE-----'
+            cert_end = '-----END CERTIFICATE-----'
+            key_content = content[content.index(key_start): content.index(key_end) + len(key_end)]
+            cert_content = content[content.index(cert_start): content.index(cert_end) + len(cert_end)]
+            save_file(key_file_name, key_content)
+            save_file(cert_file_name, cert_content)
+        except Exception as e:
+            LOG.info('Unable to store key/cert files for custom SSL certificate: %s' % e)
         if all_exist(key_file_name, cert_file_name):
             return target_file, cert_file_name, key_file_name
     if random and target_file:
